/* $Id: main.cpp,v 1.1.1.1 2005/04/22 17:18:53 ndsdev Exp $ */
/**
 * @file
 * This is part of the first tile tutorial.  Information was gather
 * from several locations to get this to work.  At the minimum, the
 * following sites were consulted to get this far:
 *
 * The full programmers guide can be found at:
 * http://www.dspassme.com/programmers_guide/
 *
 * - http://www.drunkencoders.com/
 * - http://www.webbesen.dk/gba/
 *
 * This code has been tested in both dualis realease 2 as well as on
 * an actual Nintendo DS via a PassMe device.
 *
 * PhoenixRising
 **/

/**
 * @brief
 * This doesn't seem to be defined in NDS libraries, yet the pcx2gba
 * outputs u16 arrays.
 **/
typedef unsigned short int u16;

#include "tiles.h"
#include "alpha8.h"
#include <NDS/NDS.h>
#include <NDS/ARM9/BIOS.h>

/**
 * @brief
 * The class that holds all of the data related to the application.
 * The main simply instances one of these classes and then executes it
 * via the "go" method.
 **/
class Tutorial01Tile {
public:
  /**
   * @brief
   * Mechanism to get ahold of the application.  This ensures that
   * both main and the interrupt handler can gain access to the same
   * application object without having to resort to global variables
   * and other evilness.
   *
   * @retval A reference to the application object
   **/
  static Tutorial01Tile& theApp(void);

  /**
   * @brief
   * Gets everything on the screen, and then goes to sleep, letting
   * the interrupt handler update things appropriately.
   **/
  void go(void);

  /**
   * @brief
   * Updates the display.
   **/
  void updateDisplay(void);

private:
  /**
   * @brief
   * Constructor prepares the application to run.  To access an
   * object, use theApp() to gain a reference to the application.
   *
   * It does the following:
   * - Initializes the internal class variables.
   * - Initializes the palette to black.  Not required, but makes
   *   viewing the palette in dualis easier.
   * - Configures the various registers on the DS to initialize both
   *   displays in MODE_0_2D.
   * - Finally, interrupts are enabled with the default 
   **/
  Tutorial01Tile();

  /**
   * @brief
   * Copy constructor is disabled.
   **/
  Tutorial01Tile(const Tutorial01Tile&);

  /**
   * @brief
   * Operator= is disabled.
   **/
  Tutorial01Tile& operator=(const Tutorial01Tile&);

  /**
   * @brief
   * Destructor never gets called because the application never ends.
   **/
  virtual ~Tutorial01Tile();

  /**
   * @brief
   * Update the map to display the forest scene.
   *
   * @param map The start of the map area for the screen to draw the
   *        forest scene.
   **/
  void drawMap(u16* map);
};

/**
 * @brief
 * A simple interrupt handler that accepts the IRQ_VBLANK event,
 * invokes the application updateDisplay method.
 **/
static void InterruptHandler(void) {
  if (IF & IRQ_VBLANK) {
    Tutorial01Tile& app(Tutorial01Tile::theApp());

    app.updateDisplay();

    IF = IRQ_VBLANK;
  }
}
// end InterruptHandler

#define C11 0   /**< Upper left corner of the castle. **/
#define C12 1   /**< Upper middle left of the castle. **/
#define C13 2   /**< Upper middle right of the castle. **/
#define C14 3   /**< Upper right cordner of the castle. **/
#define C21 4   /**< Lower left corner of the castle. **/
#define C22 5   /**< Lower middle left of the castle. **/
#define C23 6   /**< Lower middle right of the castle. **/
#define C24 7   /**< Lower right corner of the castle. **/
#define P11 8   /**< Upper left corner of the path. **/
#define P12 9   /**< Upper right corner of the path. **/
#define P21 12  /**< Lower left corner of the path. **/
#define P22 13  /**< Lower right cornder of the path. **/
#define T11 10  /**< Upper left corner of the tree. **/
#define T12 11  /**< Upper right corner of the tree. **/
#define T21 14  /**< Lower left corner of the tree. **/
#define T22 15  /**< Lower right cornder of the tree. **/

void Tutorial01Tile::drawMap(u16* map)
{
  u16 toDraw[] = {
    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,
    P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,C11,C12,C13,C14,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,

    P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,C21,C22,C23,C24,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,P11,P12,
    P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,P21,P22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

    T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,P11,P12,P11,P12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,T11,T12,
    T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,P21,P22,P21,P22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,T21,T22,

  };

  for(u16 i = 0; i < sizeof(toDraw) / sizeof(u16); ++i) {
    map[i] = toDraw[i];
  }
}
// end Tutorial01Tile::drawMap

// do not combine these defines (other than offset for main display)
#define VRAM_ALL_LCD       0 /**< @todo attempt to remove **/
#define VRAM_ABCD_BG       1 /**< @todo attempt to remove **/
#define VRAM_ABCD_OBJ      2 /**< @todo attempt to remove **/
#define VRAM_ABCD_TEXTURE  3 /**< @todo attempt to remove **/
 
Tutorial01Tile::Tutorial01Tile()
{
  unsigned int i, j;
  const int char_base = 0;
  const int screen_base = 20;

  // Turn on the screens and 2D cores and switch to mode 0

  POWER_CR = POWER_ALL_2D;
 
  vramSetBankA(VRAM_A_MAIN_BG);
  vramSetBankC(VRAM_C_SUB_BG);
 
  DISPLAY_CR = MODE_0_2D | DISPLAY_BG0_ACTIVE;
  SUB_DISPLAY_CR = MODE_0_2D | DISPLAY_BG0_ACTIVE;

  SUB_BG0_CR = BG_256_COLOR | BG_TILE_BASE(char_base) |
    BG_MAP_BASE(screen_base);
  BG0_CR = BG_256_COLOR | BG_TILE_BASE(char_base) | BG_MAP_BASE(screen_base);

  u16* sub_tile = (u16*)CHAR_BASE_BLOCK_SUB(char_base);
  u16* sub_map = (u16*)SCREEN_BASE_BLOCK_SUB(screen_base);
  u16* tile = (u16*)CHAR_BASE_BLOCK(char_base);
  u16* map = (u16*)SCREEN_BASE_BLOCK(screen_base);

  // Load our tile information
  for(i = 0; i < sizeof(alpha8Data) / sizeof(u16); ++i) {
    sub_tile[i] = alpha8Data[i];
  }

  for(i = 0; i < sizeof(tilesData) / sizeof(u16); ++i) {
    tile[i] = tilesData[i];
  }

  for(u16 i = 0; i < sizeof(tilesPalette) / sizeof(u16); ++i) {
    BG_PALETTE[i] = tilesPalette[i];
  }

  for(i = 0; i < sizeof(alpha8Palette) / sizeof(u16); ++i) {
    BG_PALETTE_SUB[i] = alpha8Palette[i];
  }

  BG_PALETTE_SUB[0] = RGB15(0, 0, 0);
  BG_PALETTE_SUB[1] = RGB15(0, 31, 0);

  // Tile information loaded

  char* msg[] = {
    "",
    "Tile Demo.",
    "tile01 project.",
    "",
    "http://www.dspassme.com/",
    "            programmers_guide/",
    "",
    "PhoenixRising",
  //01234567890123456789012345678901
  };

  // Draw the message on the screen.
  for(i = 0; i < sizeof(msg) / sizeof(char*); ++i) {
    for(j = 0; msg[i][j] != '\0'; ++j) {
      sub_map[j + (i * 32)] = msg[i][j] - ' ';
    }
  }

  // Draw the forest scene
  drawMap(map);
  // Both screens updated.

  // Enable the V-blank interrupt
  IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  IE = IRQ_VBLANK;
  IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  IME = 1;
}
/** end Tutorial01Tile::Tutorial01Tile */

Tutorial01Tile::~Tutorial01Tile()
{}

Tutorial01Tile& Tutorial01Tile::theApp(void)
{
  static Tutorial01Tile app;
  return app;
}

void Tutorial01Tile::go(void)
{
  while (1) {
    swiWaitForVBlank();
  }
}

void Tutorial01Tile::updateDisplay(void) {
}

/**
 * @brief
 * Gains access to the application and invokes the go method.
 *
 * @param argc Ignored
 * @param argv Ignored
 *
 * @retval 0 Returned, but never executed because the code never
 *         returns.
 **/
int main(int argc, char** argv) {
  Tutorial01Tile& app(Tutorial01Tile::theApp());

  app.go();

  return 0;
}
/* end main */
